from __future__ import unicode_literals

from email.header import Header

import django
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse
from django.views.generic.list import BaseListView, MultipleObjectMixin

from .utils import ColumnSerializer


def encode_header(value):
    return Header(value, 'utf-8').encode()


class CsvResponse(HttpResponse):
    def __init__(self, filename, content_type='text/csv', **kwargs):
        super(CsvResponse, self).__init__(content_type=content_type, **kwargs)
        disposition = 'attachment; filename="{0}"'.format(filename)
        # BBB: Django 1.4 and earlier didn't support non-ASCII headers.  Later
        # versions do this for us.
        if django.VERSION < (1, 5):
            disposition = encode_header(disposition)
        self['Content-Disposition'] = disposition


class CsvResponseMixin(MultipleObjectMixin):
    """
    A ListView mixin that returns a CsvResponse.
    """
    response_class = CsvResponse
    column_serializer_class = ColumnSerializer
    columns = None
    output_headers = True
    filename = '{model_name}_list.csv'

    def render_to_response(self, context, **kwargs):
        queryset = context['object_list']
        model = queryset.model
        response = self.response_class(
            filename=self.get_filename(model),
        )
        serialize = self.get_column_serializer(model)
        serialize(queryset, file=response)
        return response

    def get_column_serializer_class(self, model):
        return self.column_serializer_class

    def get_column_serializer(self, model):
        return self.get_column_serializer_class(model)(
            self.get_columns(model),
            output_headers=self.output_headers,
        )

    def get_columns(self, model):
        if self.columns is None:
            raise ImproperlyConfigured('Please set the columns.')
        return self.columns

    def get_filename(self, model):
        opts = model._meta
        try:
            model_name = opts.model_name
        except AttributeError:
            # for Django < 1.6. Deprecated in 1.6 & removed in 1.8.
            model_name = opts.module_name
        return self.filename.format(
            model_name=model_name,
        )


class CsvView(CsvResponseMixin, BaseListView):
    """
    A ListView that returns a CsvResponse.
    """



from functools import partial
from io import BytesIO
from operator import attrgetter

import unicodecsv as csv
from django.core.exceptions import ImproperlyConfigured


try:
    from django.utils.encoding import force_text
except ImportError:  # Django < 1.4
    from django.utils.encoding import force_unicode as force_text


def get_pretty_name(accessor):
    return accessor.replace('_', ' ') \
        .replace('.', ' ') \
        .capitalize()


def Getter(accessor, normalizer=lambda x: x):
    """
    Returns a function that will access an attribute off of an object.  If that
    attribute is callable, it will call it.  Accepts a normalizer to call on
    the value at the end.
    """
    if not callable(accessor):
        short_description = get_pretty_name(accessor)
        accessor = attrgetter(accessor)
    else:
        short_description = getattr(accessor, 'short_description', None)

    def getter(obj):
        ret = accessor(obj)
        # handle things like get_absolute_url
        if callable(ret):
            ret = ret()
        return normalizer(ret)

    if short_description:
        getter.short_description = short_description

    return getter


# Should these be i18nized?
bool2string_map = {True: 'Yes', False: 'No'}

BooleanGetter = partial(Getter, normalizer=bool2string_map.get)


def DisplayGetter(accessor, *args, **kwargs):
    """
    Returns a Getter that gets the display name for a model field with choices.
    """
    short_description = get_pretty_name(accessor)
    accessor = 'get_%s_display' % accessor
    getter = Getter(accessor, *args, **kwargs)
    getter.short_description = short_description
    return getter


class ColumnSerializer(object):
    output_headers = True

    def __init__(self, columns, **kwargs):
        self.output_headers = kwargs.get('output_headers', self.output_headers)
        self.normalized_columns = list(map(self._normalize_column, columns))

    def __call__(self, queryset, file=None):
        """
        Serializes a queryset to CSV. If you pass in a file, it will write to
        that file, otherwise, it will just return a string.
        """
        if file is None:
            output = BytesIO()
            writer = csv.writer(output)
        else:
            writer = csv.writer(file)

        if self.output_headers:
            writer.writerow(self.get_header_row())

        for obj in queryset:
            writer.writerow(self.get_row(obj))

        if file is None:
            output.seek(0)
            return output.read().decode('utf-8')

    def format_header(self, column):
        if self.output_headers:
            try:
                return column.short_description
            except AttributeError:
                raise ImproperlyConfigured(
                    "If you pass a function as an accessor,"
                    " please provide a column title."
                )

    def get_header_row(self):
        return [force_text(c[1]) for c in self.normalized_columns]

    def get_row(self, obj):
        return [force_text(c[0](obj)) for c in self.normalized_columns]

    def _normalize_column(self, column):
        # column can either be a 2-tuple of (accessor, header), or just an
        # accessor.  accessor will be passed to Getter, and we will get the
        # header off of the Getter. Returns a 2-tuple of (Getter, header).
        if isinstance(column, (tuple, list)):
            column = self._normalize_getter(column[0]), column[1]
        else:
            column = self._normalize_getter(column)
            column = (column, self.format_header(column))
        return column

    _getter_cache = {}

    def _normalize_getter(self, getter):
        if getter not in self._getter_cache:
            self._getter_cache[getter] = Getter(getter)
        return self._getter_cache[getter]



import os

from setuptools import setup


install_requires = ['Django>=1.3', 'unicodecsv']
tests_require = []


def read_file(filename):
    with open(os.path.join(os.path.dirname(__file__), filename)) as f:
        return f.read()

setup(
    name='django-separated',
    version='1.1.0',
    author="Fusionbox, Inc.",
    author_email="programmers@fusionbox.com",
    url="https://github.com/fusionbox/django-separated",
    keywords="django csv class-based views",
    description="Class-based view and mixins for handling CSV with Django.",
    long_description=read_file('README.rst'),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Topic :: Internet :: WWW/HTTP',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
    ],
    install_requires=install_requires,
    tests_require=tests_require,
    packages=[
        'separated',
    ],

    test_suite='testproject.runtests',
)
